@@ -1,4 +1,6 @@
 <?php
+require_once __DIR__ . '/../../utils.inc';
+
 class DebugClient
 {
 	// free port will be selected automatically by the operating system
@@ -33,14 +35,7 @@ class DebugClient
 
 	public function __construct()
 	{
-		$envTmpDir = getenv('TEST_TMP_DIR');
-		$this->tmpDir = $envTmpDir !== false ? $envTmpDir : sys_get_temp_dir();
-		$this->tmpDir .= '/';
-		$workerId = getenv( 'TEST_PHP_WORKER' );
-		if ( $workerId !== false )
-		{
-			$this->tmpDir .= "{$workerId}-";
-		}
+		$this->tmpDir = getTmpDir();
 	}
 
 	private function open( &$errno, &$errstr )
@@ -55,7 +50,7 @@ class DebugClient
 		return $socket;
 	}
 
-	private function launchPhp( &$pipes, $filename, array $ini_options = [] )
+	private function launchPhp( &$pipes, $filename, array $ini_options = [], array $extra_options = [] )
 	{
 		@unlink( $this->tmpDir . 'error-output.txt' );
 		@unlink( $this->tmpDir . 'remote_log.txt' );
@@ -71,13 +66,34 @@ class DebugClient
 			"xdebug.start_with_request" => "'yes'",
 			"xdebug.client_host" => $this->getIPAddress(),
 			"xdebug.client_port" => $this->getPort(),
+			'xdebug.control_socket' => "'no'",
 		);
 
+		$env_vars = array_key_exists( 'env', $extra_options ) ? $extra_options['env'] : [];
+		$env_vars += $_ENV;
+
 		$options = (getenv('TEST_PHP_ARGS') ?: '');
 		$ini_options = array_merge( $default_options, $ini_options );
 		foreach ( $ini_options as $key => $value )
 		{
-			$options .= " -d{$key}=$value";
+			if ( is_array( $value ) )
+			{
+				foreach ( $value as $valueElement )
+				{
+					$options .= " -d{$key}=$valueElement";
+				}
+			}
+			else
+			{
+				$options .= " -d{$key}=$value";
+			}
+		}
+
+		if ( array_key_exists( 'auto_prepend', $extra_options ) )
+		{
+			$prependFile = "{$this->tmpDir}auto-prepend.inc";
+			file_put_contents( $prependFile, $extra_options['auto_prepend'] );
+			$options .= " -dauto_prepend_file={$prependFile}";
 		}
 
 		$php = getenv( 'TEST_PHP_EXECUTABLE' );
@@ -87,7 +103,7 @@ class DebugClient
 		}
 		$cwd = dirname( __FILE__ );
 
-		$process = proc_open( $cmd, $descriptorspec, $pipes, $cwd );
+		$process = proc_open( $cmd, $descriptorspec, $pipes, $cwd, $env_vars );
 		return $process;
 	}
 
@@ -101,7 +117,7 @@ class DebugClient
 		return " {$m[1]}=\"file://{$fm[1]}\"";
 	}
 
-	function doRead( $conn, string $transaction_id = null )
+	function doRead( $conn, ?string $transaction_id = null )
 	{
 		stream_set_timeout( $conn, 3 );
 		do {
@@ -174,8 +190,8 @@ class DebugClient
 			echo "Address: {$this->getAddress()}\n";
 			return false;
 		}
-		$this->php = $this->launchPhp( $this->ppipes, $filename, $ini_options );
-		$conn = @stream_socket_accept( $this->socket, isset( $options['timeout'] ) ? $options['timeout'] : 20 );
+		$this->php = $this->launchPhp( $this->ppipes, $filename, $ini_options, $options );
+		$conn = @stream_socket_accept( $this->socket, isset( $options['timeout'] ) ? $options['timeout'] : 5 );
 
 		if ( $conn === false )
 		{
